#include <Wire.h>
#include <SPI.h>
#include <Sodaq_PcInt.h>

//SODAQ Mbili libraries
#include <RTCTimer.h>
#include <Sodaq_DS3231.h>

//The delay between the sensor readings 
#define READ_DELAY 60000

//Radiation PIN
#define RAD_PIN 10

//RTC Timer
RTCTimer timer;

volatile int pulse=0;
const float alpha=53.032; // cpm = uSv x alph

void setup() 
{
  //Initialise the serial connection
  Serial.begin(9600);
  
  //Initialise sensors
  setupSensors();
  
  //Setup timer events
  setupTimer();
  
  //Take first reading immediately
  takeReading(getNow());
}

void loop() 
{
  //Update the timer 
  timer.update();  
}

void takeReading(uint32_t ts)
{  
   //Create the data record
  String dataRec = createDataRecord();
  //Echo the data to the serial connection
  Serial.println(dataRec);
  pulse=0;
}

void setupSensors()
{
  //PcInt::attachInterrupt(RAD_PIN, exampleISR); //D4 Yellow Grove pin 1

   //Switch on the second Grove column
  pinMode(RAD_PIN, INPUT_PULLUP);
  digitalWrite(RAD_PIN, HIGH);
  pinMode(GROVEPWR, OUTPUT);
  digitalWrite(GROVEPWR, HIGH);
  //D10 Interrupt
  attachInterrupt(2, exampleISR, FALLING); //D10 Yellow Grove pin 1

  //Initialise the DS3231 RTC
  rtc.begin();
}


void setupTimer()
{
  //Instruct the RTCTimer how to get the current time reading
  timer.setNowCallback(getNow);

  //Schedule the reading every second
  timer.every(READ_DELAY, takeReading);
}

String createDataRecord()
{
  //Create a String type data record in csv format
  //TimeDate, TempSHT21, TempBMP, PressureBMP, HumiditySHT21
  String data = getDateTime() + ", ";
  data += String(pulse) + ", ";
  data += String(pulse/alpha);
  
  return data;
}

String getDateTime()
{
  String dateTimeStr;
  
  //Create a DateTime object from the current time
  DateTime dt(rtc.makeDateTime(rtc.now().getEpoch()));

  //Convert it to a String
  dt.addToString(dateTimeStr);
  
  return dateTimeStr;  
}

uint32_t getNow()
{
  return millis();
}

void exampleISR()
{
  pulse++;
  //Normally it is bad practice
  //to use a println() in an ISR
  //Serial.println("Int");
}
